UNKNOWN = "Unknown of class name"
FOR = "for"
BILLING_PERIOD = "For billing period"

# DLMS UA 1000-1 Ed 14 7.3.2 Value group B Table 56
INSTANCE = "instance"
CHANNEL = "channel"
UTILITY_SPECIFIC = "utility specific code"
MANUFACTURER_SPECIFIC = "manufacturer specific code"
RESERVED = "reserved"

OVER_VOLTAGE_COUNTER = "over voltage counter"


GLOBAL_METER_RESET_SCRIPT_TABLE = "Global meter reset Script table"
MDI_RESET_END_OF_BILLING_PERIOD_SCRIPT_TABLE = "MDI reset / End of billing period Script table"
TARIFFICATION_SCRIPT_TABLE = "Tariffication Script table"
SET_OUTPUT_SIGNALS_SCRIPT_TABLE = "Set output signals Script table"
DISCONNECT_CONTROL_SCRIPT_TABLE = "Disconnect control Script table"
IMAGE_ACTIVATION_SCRIPT_TABLE = "Image activation Script table"
PUSH_SCRIPT_TABLE = "Push Script table"
RU_STOP_FRAME_SCRIPT_TABLE = "Russian. Stop frame script table"
SPECIAL_DAYS_TABLE = "Special days table"
END_OF_BILLING_PERIOD_SINGLE_ACTION_SCHEDULE = "End of billing period Single action schedule"
DISCONNECT_CONTROL_SINGLE_ACTION_SCHEDULE = "Disconnect control Single action schedule"
IMAGE_ACTIVATION_SINGLE_ACTION_SCHEDULE = "Image activation Single action schedule"
OUTPUT_CONTROL_SINGLE_ACTION_SCHEDULE = "Output control Single action schedule"
PUSH_SINGLE_ACTION_SCHEDULE = "Push Single action schedule"
LOAD_PROFILE_CONTROL_SINGLE_ACTION_SCHEDULE = "Load profile control Single action schedule"
M_BUS_PROFILE_CONTROL_SINGLE_ACTION_SCHEDULE = "M-Bus profile control Single action schedule"
FUNCTION_CONTROL_SINGLE_ACTION_SCHEDULE = "Function control Single action schedule"
RU_LIMITER_BY_POWER = "RU. Limiter by power"
RU_LIMITER_BY_CURRENT = "RU. Limiter by current"
RU_LIMITER_BY_VOLTAGE = "RU. Limiter by voltage"
RU_LIMITER_BY_MAGNETIC = "RU. Limiter by magnetic"
RU_LIMITER_BY_DIFFERENCE_CURRENT = "RU. Limiter by difference current"
RU_LIMITER_BY_TEMPERATURE = "RU. Limiter by temperature"
RU_IEC_HDLC_SETUP_OPTO = "IEC HDLC Setup Opto"
RU_IEC_HDLC_SETUP_RS_485 = "IEC HDLC Setup RS-485"
RU_IEC_HDLC_SETUP_GSM = "IEC HDLC Setup GSM"
CURRENT_ASSOCIATION = "Current association"
RU_PUBLIC_CLIENT_ASSOCIATION = "Public client association"
RU_METER_READER_ASSOCIATION = "Meter reader association"
RU_UTILITY_SETTING_ASSOCIATION = "Utility setting association"
RU_PUSH = "PUSH"
COSEM_LOGICAL_DEVICE_NAME = "COSEM logical device name"
INVOCATION_COUNTER = "Invocation counter"
RU_SPECIFIC_PASSPORT_DATA_PROFILE = "Russian specific passport data profile"
RU_DEVICE_FACTORY_NUMBER = "Device factory number"
RU_DEVICE_TYPE = "Device Type"
RU_DEVICE_METROLOGICAL_VERSION = "Device metrological version"
RU_PRODUCER_NAME = "Producer name"
RU_DEVICE_RELEASE_DATE = "Device release date"
RU_REMOTE_CONSOLE_SERIAL_NUMBER = "Remote console Serial number"
RU_SPODES_VERSION = "SPODES version"
RU_DEVICE_CONNECTION_SCHEME = "Device Connection scheme"
RU_DEVICE_ID = "device ID"
RU_COUNTER_POINT_DATA = "counter point data"
I_O_CONTROL_SIGNAL_OBJECTS_GLOBAL = "I/O control signal objects, global"
# 6.2.4 Other abstract general purpose OBIS codes
# Program entries Table 60
ACTIVE_FIRMWARE_IDENTIFIER = "Active firmware identifier"
ACTIVE_FIRMWARE_VERSION = "Active firmware version"
ACTIVE_FIRMWARE_SIGNATURE = "Active firmware signature"
# 6.2.13 Register monitor and alarm monitor objects
RU_ALARM_MONITOR_1 = "RU. Load alarm monitor"
# 6.2.19 Standard readout profile objects
GENERAL_DISPLAY_READOUT = "General display readout"
ALTERNATE_DISPLAY_READOUT = "Alternate display readout"
# 6.2.44 Parameter changes and calibration objects
PARAMETER_CHANGES_AND_CALIBRATION_0 = "RU. Correction counter(Configuring)"
# 6.2.48 Status of internal control signals objects
RU_LCD_BACKLIGHT_MODE = "Lcd backlight mode"

POWER_QUALITY_STATUS_DAILY = "Power quality status (profile of daily testimony)"
POWER_QUALITY_STATUS_QUALITY_LOG = "Power quality status (power quality log)"
TIME_OF_OPERATION = "Time of operation"
RU_DURATION_OF_FAILURE_OVERSTRAIN = "Duration of failure/overstrain"
RU_AMBIENT_TEMPERATURE = "Ambient temperature"
EVENTS_RELATED_TO_VOLTAGE = "Events related to voltage"
RU_EVENTS_RELATED_TO_CURRENT = "Events related to current"
RU_EVENTS_RELATED_TO_LOAD_RELAY = "Events related to load relay"
RU_EVENTS_FOR_PROGRAMMING_DEVICE_PARAMETERS = "Events for programming PU parameters"
RU_EXTERNAL_IMPACT_EVENTS = "External impact events"
RU_COMMUNICATION_EVENTS = "Communication events"
RU_ACCESS_CONTROL_EVENTS = "Access control events"
RU_EVENT_CODES_FOR_THE_SELF_DIAGNOSIS_LOG = "Event codes for the self-diagnosis log"
RU_EVENTS_FOR_EXCEEDING_THE_REACTIVE_POWER = "Events for exceeding the reactive power"
RU_CHANNEL_NUMBER_INTERFACE = "channel number (interface)"
COMMUNICATION_ADDRESS = "Communication address"
LOAD_PARAMETERS_PROFILE = "Load parameters profile"
# 6.2.59 Event counter objects
RU_RELAY_TRIGGERING_METER_FOR_OPENING = "RU. Relay triggering meter for opening"
# 6.2.62 Meter tamper event related objects
METER_OPEN_EVENT_COUNTER = "Meter open event counter"
METER_OPEN_EVENT_TIME_STAMP = "Meter open event, time stamp of current event occurrence"
METER_OPEN_EVENT_DURATION = "Meter open event, duration of current event"
METER_OPEN_EVENT_CUMULATIVE_DURATION = "Meter open event, cumulative duration "
TERMINAL_COVER_OPEN_EVENT_COUNTER = "Terminal cover open event counter"
TERMINAL_COVER_OPEN_EVENT_TIME_STAMP = "Terminal cover open event, time stamp of current event occurrence"
TERMINAL_COVER_OPEN_EVENT_DURATION = "Terminal cover open event, duration of current event"
TERMINAL_COVER_OPEN_EVENT_CUMULATIVE = "Terminal cover open event, cumulative duration"
TILT_EVENT_COUNTER = "Tilt event counter"
TILT_EVENT_TIME_STAMP = "Tilt event, time stamp of current event occurrence"
TILT_EVENT_DURATION = "Tilt event, duration of current event"
TILT_EVENT_CUMULATIVE_DURATION = "Tilt event, cumulative duration"
STRONG_DC_MAGNETIC_FIELD_EVENT_COUNTER = "Strong DC magnetic field event counter"
STRONG_DC_MAGNETIC_FIELD_EVENT_TIME_STAMP = "Strong DC magnetic field event, time stamp of current event occurrence"
STRONG_DC_MAGNETIC_FIELD_EVENT_DURATION = "Strong DC magnetic field event, duration of current event"
STRONG_DC_MAGNETIC_FIELD_EVENT_CUMULATIVE_DURATION = "Strong DC magnetic field event, cumulative duration"
SUPPLY_CONTROL_SWITCH_VALVE_TAMPER_EVENT_COUNTER = "Supply control switch / valve tamper event counter"
SUPPLY_CONTROL_SWITCH_VALVE_TAMPER_EVENT_TIME_STAMP = "Supply control switch / valve tamper event, time stamp of current event occurrence"
SUPPLY_CONTROL_SWITCH_VALVE_TAMPER_EVENT_DURATION = "Supply control switch / valve tamper event, duration of current event"
SUPPLY_CONTROL_SWITCH_VALVE_TAMPER_EVENT_CUMULATIVE_DURATION = "Supply control switch / valve tamper event, cumulative duration"
METROLOGY_TAMPER_EVENT_COUNTER = "Metrology tamper event counter"
METROLOGY_TAMPER_EVENT_TIME_STAMP = "Metrology tamper event, time stamp of current event occurrence"
METROLOGY_TAMPER_EVENT_DURATION = "Metrology tamper event, duration of current event"
METROLOGY_TAMPER_EVENT_CUMULATIVE_DURATION = "Metrology tamper event, cumulative duration"
COMMUNICATION_TAMPER_EVENT_COUNTER = "Communication tamper event counter"
COMMUNICATION_TAMPER_EVENT_TIME_STAMP = "Communication tamper event, time stamp of current event occurrence"
COMMUNICATION_TAMPER_EVENT_DURATION = "Communication tamper event, duration of current event"
COMMUNICATION_TAMPER_EVENT_CUMULATIVE_DURATION = "Communication tamper event, cumulative duration"
# 6.2.64 Alarm register, Alarm filter and Alarm descriptor objects
RU_ALARM_REGISTER_1 = "RU. Alarm register. Initiative output"
RU_ALARM_FILTER_1 = "RU. Alarm filter. Initiative exit"
RU_ALARM_REGISTER_2 = "RU. Alarm register. Load relay"
RU_ALARM_FILTER_2 = "RU. Alarm filter. Load relay"
# СТО 34.01-5.1-006-2021 Д.11 Журналы событий и захватываемые параметры
RU_VOLTAGE_LOG = "RU. Voltage log"
RU_CURRENT_LOG = "RU. Current log"
RU_COMMUTATION_LOG = "RU. Commutation log"
RU_DATA_CORRECTION_LOG = "RU. Data correction log"
RU_EXTERNAL_IMPACT_LOG = "RU. External impact log"
RU_COMMUNICATION_LOG = "RU. Communication log"
RU_ACCESS_LOG = "RU. Access log"
RU_SELF_DIAGNOSTIC_LOG = "RU. Self diagnostic log"
RU_REACTIVE_POWER_LOG = "RU. Reactive power log"
RU_QUALITY_POWER_LOG = "RU. Quality power log"
RU_STATUS_I_O_LOG = "RU. Status I/O log"
RU_REACTIVE_POWER_LIMIT_LOG = "RU. Reactive power limit log"
RU_TIME_CORRECTION_LOG = "RU. Time correction log"
RU_START_YEAR_LOG = "RU. Start year log"
RU_QUALITY_FOR_CALCULATION_PERIOD_LOG = "RU. Powerline quality for calculation period log"
RU_CONTROL_POWER_LOG = "RU. Control power log"
RU_BATTERY_LOG = "RU. Battery log"
RU_CONTROL_OF_LOAD_RELAY_BLOCKER_LOG = "RU. Control of load relay blocker log"
RU_TEMPERATURE_CONTROL_LOG = "RU. Temperature control log"
RU_VOLTAGE_DEVIATION_LOG = "RU. Voltage deviation log"
RU_LINEAR_VOLTAGE_DEVIATION_LOG = "RU. Linear voltage deviation log"
RU_ABNORMAL_NETWORK_SITUATION_LOG = "RU. Abnormal network situation log"
RU_VOLTAGE_INTERRUPTION_LOG = "RU. Over interruption log"
RU_OVER_VOLTAGE_LOG = "RU. Over voltage log"
#
ITE_FIRMWARE_DESCRIPTOR = "ITE. Firmware descriptor"
ITE_MAGNETIC_SENSOR_STATUS = "ITE. Magnetic sensor status"
ITE_DISCRETE_OUTPUTS = "ITE. Discrete outputs"
ITE_SETTING_OF_RELAY_INCLUSION_PER_DAY = "ITE. Setting of relay inclusion per day"
ITE_SETTINGS_MESSAGES = "ITE. Settings messages"
ITE_CORE_REGISTERS = "ITE. Core registers"
ITE_BLE_ID = "ITE. BLUETOOTHLE ID"
ITE_BITMAP = "ITE. BITMAP loader"
ITE_ICCID = "ITE. ICCID"
RU_BODY_OPENING_STATE = "RU. Current condition of the opening sensor of the body"
RU_TERMINALS_COVER_OPENING_STATE = "RU. Current condition of the opening sensor of the terminal covers"
RU_MAGNETIC_FIELD_STATE = "RU. Current state of the magnetic field sensor"
RU_HF_FIELD_STATE = "RU. Current state of HF field"
RU_ELECTRONIC_SEALS_FIXED_STATE_OF_EVENTS = "RU. Fixed state of events of electronic seals"
RU_PRESSING_ELECTRONIC_SEALS = "RU. Pressing electronic seals"
RU_CLEAR_OF_ELECTRONIC_SEALS_FIXED_STATE = "RU. Cleaning recorded events (magnet and HF field)"
RU_FIRST_OPENING_TIME_OF_BODY = "RU. The first opening time of the electronic seal of the case"
RU_FIRST_OPENING_TIME_OF_TERMINALS_COVER = "RU. The first opening time of the electronic seal of the terminal covers"
# 7.4. Abstract object
# 7.4.1 Table 60 General and service entry object - Abstract
RU_RELAY_LOAD_ARBITRATOR = "RU. Relay load arbitrator"
# Communication port log parameters
ITE_USED_COMMUNICATION_INTERFACES = "ITE. Used communication interfaces"
# Consumer messages
LOCAL_CONSUMER_MESSAGE = "Consumer message via local consumer information port"
DISPLAY_CONSUMER_MESSAGE = "Consumer message via the meter display and / or via consumer information port"
# 7.4.5 Data profile objects – Abstract
GSM_DIAGNOSTIC_PROFILE = "GSM diagnostic profile"
# electricity
# 7.5.1 Value group C codes – Electricity
ACTIVE_POWER_PLUS = "Active power+ (QI+QIV)"
ACTIVE_POWER_MINUS = "Active power- (QII+QIII)"
REACTIVE_POWER_PLUS = "Reactive power+ (QI+QII)"
REACTIVE_POWER_MINUS = "Reactive power- (QIII+QIV)"
REACTIVE_POWER_QI = "Reactive power QI"
REACTIVE_POWER_QII = "Reactive power QII"
REACTIVE_POWER_QIII = "Reactive power QIII"
REACTIVE_POWER_QIV = "Reactive power QIV"
APPARENT_POWER_PLUS = "Apparent power+ (QI+QIV)"
APPARENT_POWER_MINUS = "Apparent power- (QII+QIII)"
CURRENT = "Current"
VOLTAGE = "Voltage"
POWER_FACTOR = "Power factor"
SUPPLY_FREQUENCY = "Supply frequency"
ACTIVE_POWER_ABS_PLUS = "Active power (abs(QI+QIV)+(abs(QII+QIII))"
ACTIVE_POWER_ABS_MINUS = "Active power (abs(QI+QIV)-abs(QII+QIII))"
ACTIVE_POWER_QI = "Active power QI"
ACTIVE_POWER_QII = "Active power QII"
ACTIVE_POWER_QIII = "Active power QIII"
ACTIVE_POWER_QIV = "Active power QIV"
REACTIVE_FACTOR = "Reactive factor"
CUMULATIVE = "Cumulative"
ANY_PHASE = "Any phase"
ALL_PHASE = "All phase"
L1 = "L1"
L2 = "L2"
L3 = "L3"
L1_L2 = "L1-L2"
L2_L3 = "L2-L3"
L3_L1 = "L3-L1"
CUMULATIVE_AMPERE_SQUARED_HOURS = "Cumulative Ampere-squared hours (QI+QII+QIII+QIV)"
CUMULATIVE_VOLT_SQUARED_HOURS = "Cumulative Volt-squared hours (QI+QII+QIII+QIV)"
L0_CURRENT_NEUTRAL = "L0 current (neutral)"
RU_DIFFERENTIAL_CURRENT = "RU. Differential current"
PERCENT = "Percent"
RU_LINEAR_VOLTAGE = "RU. Linear voltage"
RU_POSITIVE_DEVIATION = "RU. Positive deviation"
RU_NEGATIVE_DEVIATION = "RU. Negative deviation"
POWER_REACTIVE_FACTOR = "Power reactive factor"
VOLTAGE_ASYMMETRY_COEFFICIENT = "RU. Voltage asymmetry coefficient"
# 7.5.2 Value group D codes – Electricity
# 7.5.2.1 Processing of measurement values
BILLING_PERIOD_AVERAGE_SINCE_LAST_RESET = "Billing period average (since last reset)"
CUMULATIVE_MINIMUM_1 = "Cumulative minimum 1"
CUMULATIVE_MAXIMUM_1 = "Cumulative maximum 1"
MINIMUM_1 = "Minimum 1"
CURRENT_AVERAGE_1 = "Current average 1"
LAST_AVERAGE_1 = "Last average 1"
MAXIMUM_1 = "Maximum 1"
INSTANTANEOUS_VALUE = "Instantaneous value"
TIME_INTEGRAL_1 = "Time integral 1"
TIME_INTEGRAL_2 = "Time integral 2"
TIME_INTEGRAL_3 = "Time integral 3"
CUMULATIVE_MINIMUM_2 = "Cumulative minimum 2"
CUMULATIVE_MAXIMUM_2 = "Cumulative maximum 2"
MINIMUM_2 = "Minimum 2"
CURRENT_AVERAGE_2 = "Current average 2"
LAST_AVERAGE_2 = "Last average 2"
MAXIMUM_2 = "Maximum 2"
TIME_INTEGRAL_7 = "Time integral 7"
TIME_INTEGRAL_8 = "Time integral 8"
TIME_INTEGRAL_9 = "Time integral 9"
TIME_INTEGRAL_10 = "Time integral 10"
CUMULATIVE_MINIMUM_3 = "Cumulative minimum 3"
CUMULATIVE_MAXIMUM_3 = "Cumulative maximum 3"
MINIMUM_3 = "Minimum 3"
CURRENT_AVERAGE_3 = "Current average 3"
LAST_AVERAGE_3 = "Last average 3"
MAXIMUM_3 = "Maximum 3"
CURRENT_AVERAGE_5 = "Current average 5"
CURRENT_AVERAGE_6 = "Current average 6"
TIME_INTEGRAL_5 = "Time integral 5"
TIME_INTEGRAL_6 = "Time integral 6"
UNDER_LIMIT_THRESHOLD = "Under limit threshold"
UNDER_LIMIT_OCCURRENCE_COUNTER = "Under limit occurrence counter"
UNDER_LIMIT_DURATION = "Under limit duration"
UNDER_LIMIT_MAGNITUDE = "Under limit magnitude"
OVER_LIMIT_THRESHOLD = "Over limit threshold"
OVER_LIMIT_OCCURRENCE_COUNTER = "Over limit occurrence counter"
OVER_LIMIT_DURATION = "Over limit duration"
OVER_LIMIT_MAGNITUDE = "Over limit magnitude"
MISSING_THRESHOLD = "Missing threshold"
MISSING_OCCURRENCE_COUNTER = "Missing occurrence counter"
MISSING_DURATION = "Missing duration"
MISSING_MAGNITUDE = "Missing magnitude"
TIME_THRESHOLD_FOR_UNDER_LIMIT = "Time threshold for under limit"
TIME_THRESHOLD_FOR_OVER_LIMIT = "Time threshold for over limit"
TIME_THRESHOLD_FOR_MISSING_MAGNITUDE = "Time threshold for missing magnitude"
CONTRACTED_VALUE = "Contracted value"
AVERAGE_VALUE_FOR_RECORDING_INTERVAL_1 = "Average value for recording interval 1"
AVERAGE_VALUE_FOR_RECORDING_INTERVAL_2 = "Average value for recording interval 2"
MINIMUM_FOR_RECORDING_INTERVAL_1 = "Minimum for recording interval 1"
MINIMUM_FOR_RECORDING_INTERVAL_2 = "Minimum for recording interval 2"
MAXIMUM_FOR_RECORDING_INTERVAL_1 = "Maximum for recording interval 1"
MAXIMUM_FOR_RECORDING_INTERVAL_2 = "Maximum for recording interval 2"
TEST_AVERAGE = "Test average"
CURRENT_AVERAGE_4_FOR_HARMONICS_MEASUREMENT = "Current average 4 for harmonics measurement"
TIME_INTEGRAL_4 = "Time integral 4"
RU_TOTAL_DEVIATION_TIME = "RU. Total deviation time"
# 7.5.3.2 Tariff rates
TOTAL = "Total"
RATE = "Rate"
MANUFACTURER_SPECIFIC_CODES = "Manufacturer specific codes"

# 7.5.3.3 Harmonics
TOTAL_FUND_ALL = "Total (fundamental + all harmonics)"
HARMONIC = "harmonic"
THD = "Total Harmonic Distortion(THD)"
TDD = "Total Demand Distortion (TDD)"
ALL_HARMONICS = "All harmonics"
ALL_HARMONICS_TO_NOMINAL_VALUE_RATIO = "All harmonics to nominal value ratio"

# 7.5.3.4 Phase angles
ANGLE_FROM = "Angle from"
TO = "to"
U_L1 = "U(L1)"
U_L2 = "U(L2)"
U_L3 = "U(L3)"
ERROR = "error"
I_L1 = "I(L1)"
I_L2 = "I(L2)"
I_L3 = "I(L3)"
I_L0 = "I(L0)"

# 7.5.5.1 Table 72 General and service entry objects – Electricity
TRANSFORMER_RATIO = "transformer ratio"
COMPLETE_COMBINED_ELECTRICITY_ID = "Complete combined Electricity ID"
# Program entries
""" init in abstract Program entries"""
# Output pulse values or constants
ACTIVE_ENERGY_OUTPUT_PULSE = "Active energy, output pulse"
REACTIVE_ENERGY_OUTPUT_PULSE = "Reactive energy, output pulse"
# Ratios
TRANSFORMER_RATIO_CURRENT = F"{TRANSFORMER_RATIO} - current (numerator)"
TRANSFORMER_RATIO_VOLTAGE = F"{TRANSFORMER_RATIO} - voltage (numerator)"
# Nominal values
NOMINAL_VOLTAGE = "Nominal voltage"
NOMINAL_CURRENT = "Nominal current"
NOMINAL_FREQUENCY = "Nominal frequency"
MAXIMUM_CURRENT = "Maximum current"
REFERENCE_VOLTAGE_FOR_POWER_QUALITY_MEASUREMENT = "Reference voltage for power quality measurement"
REFERENCED_VOLTAGE_FOR_AUX_POWER_SUPPLY = "Reference voltage for aux. power supply"
# Measurement period- / recording interval- / billing period duration
RECORDING_INTERVAL_1_FOR_LOAD_PROFILE = "Recording interval 1, for load profile"
RECORDING_INTERVAL_2_FOR_LOAD_PROFILE = "Recording interval 2, for load profile"
# Coefficients
TRANSFORMER_MAGNETIC_LOSSES = "Transformer magnetic losses"
TRANSFORMER_IRON_LOSSES = "Transformer iron losses"
LINE_RESISTANCE_LOSSES = "Line resistance losses"
LINE_REACTANCE_LOSSES = "Line reactance losses"
# 7.5.5.3 List objects – Electricity
RU_MONTHLY_PROFILE = "RU. Monthly profile"
RU_DAILY_PROFILE = "RU. Daily profile"

# 7.5.5.4 Data profile objects – Electricity
RU_LOAD_PROFILE = "RU. Load profile"


# Country specific identifiers. Russian profiles
COUNTRY_SPECIFIC_IDENTIFIER = "Country specific identifier"
RU_PROFILE_OF_CURRENT_VALUES = "RU. Profile of current values"
RU_SCALE_PROFILE_FOR_THE_MAGAZINE_OF_MONTHLY_INDICATIONS = "RU. Scale profile for the magazine of monthly indications"
RU_SCALE_PROFILE_FOR_A_JOURNAL_OF_DAILY_INDICATION = "RU. Scale profile for a journal of daily indication"
RU_SCALE_PROFILE_FOR_CURRENT_FRAMES_OF_CURRENT_VALUES = "RU. Scale profile for current-frames of current values"
RU_SCALE_PROFILE_FOR_LOAD_PROFILES = "RU. Scale profile for load profiles"
RU_TELEMECHANICS_PROFILE_FOR_TELEVISION_MEASUREMENTS = "RU. Telemechanics profile for television measurements"
RU_TELEMECHANICS_PROFILE_OF_TELEVISION_SIGNALING = "RU. Telemechanics profile of television signaling"
RU_CHANGE_LIMIT_LEVEL = "RU. Change limit level"
# ITE
ITE_CALIBRATION_STATUS = "ITE. Calibration status"
ITE_CALIBRATION_APPARENT_POWER = "ITE. Calibration. Apparent power"
ITE_CALIBRATION_ACTIVE_POWER = "ITE. Calibration. Active power"
ITE_CALIBRATION_REACTIVE_POWER = "ITE. Calibration. Reactive power"
ITE_CALIBRATION_VOLTAGE = "ITE. Calibration. Voltage"
ITE_CALIBRATION_CURRENT = "ITE. Calibration. Current"
ITE_CALIBRATION_ANGLE = "ITE. Calibration. Angle"
ITE_DISPLAY_SETTING_1 = "ITE. Display setting #1"
ITE_DISPLAY_SETTING_2 = "ITE. Display setting #2"
ITE_CLOCK_OFFSET_SETTING = "ITE. Clock offset setting"
ITE_FACTORY_SETTING_10 = "ITE Factory setting #10"
ITE_FACTORY_SETTING_11 = "ITE Factory setting #11"
ITE_FACTORY_SETTING_12 = "ITE Factory setting #12"
ITE_FACTORY_SETTING_13 = "ITE Factory setting #13"
ITE_FACTORY_SETTING_14 = "ITE Factory setting #14"
ITE_FACTORY_SETTING_15 = "ITE Factory setting #15"
ITE_FACTORY_SETTING_16 = "ITE Factory setting #16"
ITE_FACTORY_SETTING_17 = "ITE Factory setting #17"
ITE_FACTORY_SETTING_18 = "ITE Factory setting #18"
ITE_FACTORY_SETTING_19 = "ITE Factory setting #19"

# RU. Simple OBIS. СТО 34.01-5.1-006-2021.
RU_TOTAL_VOLTAGE_DEVIATION_TIME_FOR_CALCULATED_PERIOD = "RU. Total voltage deviation time for calculated period"
RU_LOAD_LOCK_STATUS = "RU. Load lock status"  # 13.5.6
# 13.9. Контроль чередования фаз
SPODES3_PHASE_ALTERNATING_CONTROL = "RU. SPODES3. Phase alternating control"
SPODES3_NOT_METROLOGICAL_FIRMWARE_ID = "RU. SPODES3. Not metrological firmware id"


# 7.2.4 Summary of rules for manufacturer, utility, consortia and country specific codes. Table 54
# KPZ - Kurgansky Priborostroitelny Zavod
KPZ_INTERFACE_LIST = "KPZ. Interface list"
