SCALER = "Масштаб"
UNIT = "Ед. измерения"
DAY_ID = "Идентификатор дня"
DAY_SCHEDULE = "Расписание дня"
START_TIME = "Время старта"
STOP_TIME = "Время окончания"
END_TIME = "Время окончания"
SCRIPT_LOGICAL_NAME = "Логическое имя скрипта"
SCRIPT_SELECTOR = "Селектор скрипта"
SEASON_PROFILE_NAME = "Имя профиля сезона"
SEASON_START = "Начало сезона"
WEEK_NAME = "Имя недели"
WEEK_PROFILE_NAME = "Имя профиля недели"
MONDAY = "понедельник"
TUESDAY = "вторник"
WEDNESDAY = "среда"
THURSDAY = "четверг"
FRIDAY = "пятница"
SATURDAY = "суббота"
SUNDAY = "воскресенье"
ITE_OPTICAL_PORT_NAME = "Имя оптопорта"
ITE_RS485_NAME = "Имя RS-485"
PRESET_TIME = "Заданное время"
VALIDITY_INTERVAL_START = "Начало интервала действия"
VALIDITY_INTERVAL_END = "Конец интервала действия"
PRECEDENCE = "Приоритет"
DELAY = "Задержка"
RELIABILITY = "Надежность"
PEAK_THROUGHPUT = "Пиковая пропускная способность"
MEAN_THROUGHPUT = "Средняя пропускная способность"
DEFAULT = "По умолчанию"
REQUESTED = "По запросу"
CELL_ID = "Идентификатор звонка"
SIGNAL_QUALITY = "Качество сигнала"
LOCATION_ID = "Идентификатор местоположения"
BER = "Ошибка битов"
IMAGE_BLOCK_NUMBER = "Номер блока"
IMAGE_BLOCK_VALUE = "Данные блока"
IMAGE_TO_ACTIVATE_SIZE = "Размер данных"
IMAGE_TO_ACTIVATE_IDENTIFICATION = "Идентификация данных"
IMAGE_TO_ACTIVATE_SIGNATURE = "Подпись данных"
IMAGE_IDENTIFIER = "Идентификатор образа"
IMAGE_SIZE = "размер образа"
ACTION_OVER_THRESHOLD = "Действие выше порога"
ACTION_UNDER_THRESHOLD = "Действие ниже порога"
EMERGENCY_PROFILE_ID = "Идентификатор аварийного профиля"
EMERGENCY_ACTIVATION_TIME = "Аварийное время активации"
EMERGENCY_DURATION = "Продолжительность аварии"
CLASS_ID = "ИК"
LOGICAL_NAME = "Логическое имя объекта"
ATTRIBUTE_INDEX = "Индекс атрибута"
DATA_INDEX = "Индекс данных"
FROM_DATE = "С даты"
TO_DATE = "До даты"
FROM_ENTRY = "С записи"
TO_ENTRY = "До записи"
FROM_SELECTED_VALUE = "От выбранного значения"
TO_SELECTED_VALUE = "До выбранного значения"
RESTRICTING_OBJECT = "Ограничивающий объект"
FROM_VALUE = "От значения"
TO_VALUE = "До значения"
SELECTED_VALUES = "Выбранные объекты"
FIRST_INDEX = "Первый индекс"
LAST_INDEX = "Последний индекс"
FIRST_INDEX_A = "Первый индекс A"
LAST_INDEX_A = "Последний индекс A"
FIRST_INDEX_B = "Первый индекс B"
LAST_INDEX_B = "Последний индекс B"
INDEX = "Индекс"
ENABLE = "Включено"
SWITCH_TIME = "Время переключения"
VALIDITY_WINDOW = "Окно валидности"
EXEC_WEEKDAYS = "Дни недели"
EXEC_SPEC_DAYS = "Спец дни"
BEGIN_DATE = "Дата начала"
END_DATE = "Дата окончания"
SERVICE_ID = "Тип сервиса"
PARAMETER = "Параметр"
SCRIPT_IDENTIFIER = "Идентификатор сценария"
ACTIONS = "Действия"
TIME = "Время"
DATE = "Дата"
SPECIAL_DAY_DATE = "Дата спец дня"
REQUEST = "Запрос"
RESPONSE = "Ответ"
DELAY_AFTER_RESPONSE = "Задержка после ответа"
ATTRIBUTE_ACCESS = "Доступ к атрибуту"
METHOD_ACCESS = "Доступ к методу"
JOINT_ISO_CTT_ELEMENT = "Совместный элемент ISO CTT"
COUNTRY_ELEMENT = "Элемент страны"
COUNTRY_NAME_ELEMENT = "Элемент названия страны"
IDENTIFIED_ORGANIZATION_ELEMENT = "Идентифицированный элемент организации"
DLMS_UA_ELEMENT = "Элемент DLMS UA"
APPLICATION_CONTEXT_ELEMENT = "Элемент контекста приложения"
CONTEXT_ID_ELEMENT = "Элемент идентификатора контекста"
CLIENT_SAP = "клиент SAP"
SERVER_SAP = "сервер SAP"
ATTRIBUTE_ID = "Индекс атрибута"
ACCESS_MODE = "Режим доступа"
ACCESS_SELECTORS = "Селекторы доступа"
AUTHENTICATION_MECHANISM_NAME_ELEMENT = "Элемент контекста приложения"
MECHANISM_ID_ELEMENT = "Идентификационный элемент механизма"
METHOD_ID = "Индекс метода"
VERSION = "Версия"
ACCESS_RIGHTS = "Права доступа"
CONFORMANCE = "Согласование"
MAX_RECEIVE_PDU_SIZE = "Максимальный размер приемного блока pdu"
MAX_SEND_PDU_SIZE = "Максимальный размер отправки pdu"
DLMS_VERSION_NUMBER = "Номер версии DLMS"
QUALITY_OF_SERVICE = "Качество обслуживания"
CYPHERING_INFO = "Информация о шифровании"
KEY_ID = "Идентификатор ключа"
KEY_WRAPPED = "Обертка ключа"
KEY_DATA = "Данные ключа"
CERTIFICATE_ENTITY = "Объект сертификата"
CERTIFICATE_TYPE = "Тип сертификата"
SERIAL_NUMBER = "Серийный номер"
ISSUER = "Издатель"
SUBJECT = "Субъект"
SUBJECT_ALT_NAME = "Альтернативное имя субъекта"
SYSTEM_TITLE = "Системное название"
CERTIFICATE_IDENTIFICATION_TYPE = "Тип идентификации сертификата"
CERTIFICATION_IDENTIFICATION_OPTIONS = "Вариант идентификации сертификата"
REQUEST_ACTOR = "Запрос актуатора"
REQUEST_ACTION_LIST = "Список запросов действий"
ACTION_UP = "Действие вверх"
ACTION_DOWN = "Действие вниз"
ITE_BIT_MAP = "BitMap данные"
RESTRICTION = "Oграничениe"
RESTRICTION_TYPE = "Тип ограничения"
RESTRICTION_VALUE = "Значение ограничения"
COLUMN = "Столбец"
TRANSPORT_SERVICE = "транспортный сервис"
DESTINATION = "адресат"
MESSAGE = "сообщение"
REPETITION_DELAY_MIN = "задержка повтора мин"
REPETITION_DELAY_EXPONENT = "задержка повтора экспонента"
REPETITION_DELAY_MAX = "задержка повтора макс"
CONFIRMATION_START_DATE = "Дата начала подтверждения"
CONFIRMATION_INTERVAL = "Интервал подтверждения"
KEK_ID = "kek идентификатор"
KEY_PARAMETERS = "параметры ключа"
KEY_CIPHERED_DATA = "Ключ зашифрованных данных"
KEY_INFO_TYPE = "тип ключa"
KEY_INFO_OPTIONS = "опции ключа"
KEY_INFO = "информация о ключе"
TRANSACTION_ID = "ID транзакции"
ORIGINATOR_SYSTEM_TITLE = "Название системы создателя"
RECIPIENT_SYSTEM_TITLE = "Название системы получателя"
OTHER_INFORMATION = "Дополнительная информация"
PROTECTION_TYPE = "Тип защиты"
PROTECTION_OPTIONS = "Опции защиты"
